void plot(const char *fname="result.root")
{
  TFile *inFile = TFile::Open(fname);

  // constants for arrays 
  const int kNChan = 32;
  const int kNQR = 26;
  const int kNVRef = 6;
  const int kNACurr = 5;
  const int kNSync = 8;

  TTree *tree = (TTree*) inFile->Get("tree");
  TCanvas *c1 = new TCanvas("c1","c1");

  //Declaration of leaves types
  Char_t          filename[200];
  Char_t          datetime[100];
  Char_t          cQR[26];
  Int_t           iChip;
  Int_t           iSta;
  Float_t         fVRef[6];
  Float_t         fVCurr[5];
  Float_t         fACurr[5];
  Float_t         fGain20mV[32];
  Float_t         fGain30mV[32];
  Float_t         fRT20mV[32];
  Float_t         fRT30mV[32];
  Float_t         fPed20mV[32];
  Float_t         fPed30mV[32];
  Float_t         fRms20mV[32];
  Float_t         fRms30mV[32];
  Float_t         fPedAna20mV[32];
  Float_t         fPedAna30mV[32];
  Float_t         fRmsAna20mV[32];
  Float_t         fRmsAna30mV[32];
  Float_t         fX20mVeven;
  Float_t         fX20mVodd;
  Float_t         fX30mVeven;
  Float_t         fX30mVodd;
  Int_t           iSync[8];
  Int_t           iDFT;
  Int_t           iJtag;
  Int_t           iMem;
  Float_t         fClk;
  Int_t           i2cErr;
  Int_t           iRCode;

  // Set branch addresses.
  tree->SetBranchAddress("filename",filename);
  tree->SetBranchAddress("datetime",datetime);
  tree->SetBranchAddress("cQR",&cQR);
  tree->SetBranchAddress("iChip",&iChip);
  tree->SetBranchAddress("iSta",&iSta);
  tree->SetBranchAddress("fVRef",fVRef);
  tree->SetBranchAddress("fVCurr",fVCurr);
  tree->SetBranchAddress("fACurr",fACurr);
  tree->SetBranchAddress("fGain20mV",fGain20mV);
  tree->SetBranchAddress("fGain30mV",fGain30mV);
  tree->SetBranchAddress("fRT20mV",fRT20mV);
  tree->SetBranchAddress("fRT30mV",fRT30mV);
  tree->SetBranchAddress("fPed20mV",fPed20mV);
  tree->SetBranchAddress("fPed30mV",fPed30mV);
  tree->SetBranchAddress("fRms20mV",fRms20mV);
  tree->SetBranchAddress("fRms30mV",fRms30mV);
  tree->SetBranchAddress("fPedAna20mV",fPedAna20mV);
  tree->SetBranchAddress("fPedAna30mV",fPedAna30mV);
  tree->SetBranchAddress("fRmsAna20mV",fRmsAna20mV);
  tree->SetBranchAddress("fRmsAna30mV",fRmsAna30mV);
  tree->SetBranchAddress("fX20mVeven",&fX20mVeven);
  tree->SetBranchAddress("fX20mVodd",&fX20mVodd);
  tree->SetBranchAddress("fX30mVeven",&fX30mVeven);
  tree->SetBranchAddress("fX30mVodd",&fX30mVodd);
  tree->SetBranchAddress("iSync",iSync);
  tree->SetBranchAddress("iDFT",&iDFT);
  tree->SetBranchAddress("iJtag",&iJtag);
  tree->SetBranchAddress("iMem",&iMem);
  tree->SetBranchAddress("fClk",&fClk);
  tree->SetBranchAddress("i2cErr",&i2cErr);
  tree->SetBranchAddress("iRCode",&iRCode);

  Long64_t nentries = tree->GetEntries();
  cout << " nentries = " << nentries << endl;

  const int kNHistMax = 50;
  TH1F *h[kNHistMax];
  int nbins[kNHistMax] = { 100 };
  float min[kNHistMax] = { 0 };
  float max[kNHistMax] = { 100 };

  // 
  vector <string> hTitle;
  vector <string> hFill;

  int nHists = 0;
  hTitle.push_back("VDDA (V)"); min[nHists] = 1.2; max[nHists] = 1.3; hFill.push_back("fVRef[0]"); nHists++;
  hTitle.push_back("VREF (V)"); min[nHists] = 1.07; max[nHists] = 1.11; hFill.push_back("fVRef[1]"); nHists++;

  // Print Strings stored in Vector
  for (int i=0; i<hTitle.size(); i++)    
    cout << hId[i] << "\n" << hTitle[i] << "\n" << hFill[i] << "\n"; 

  /*
  vector <string> hId;
  char id[50];
  hId.push_back( Form("h%d", nHists) ); 

  // voltages and currents plots
  const char *VRefText[] = { "VDDA (V)", "VREF (V)", "VDD (V)", "V450 (V)", "V600 (V)", "V750 (V)" };
  const char *VCurrText[] = { "FE1_CURR (V)", "FE2_CURR (V)", "AD_CURR (V)", "DR_CURR (V)", "DG_CURR (V)" };
  const char *ACurrText[] = { "FE1_CURR (mA)", "FE2_CURR (mA)", "AD_CURR (mA)", "DR_CURR (mA)", "DG_CURR (mA)" };

  // limits for plots
  float VRefMin[kNVRef] = {1.2, 1.07, 1.2, 0.45, 0.6, 0.75};
  float VRefMax[kNVRef] = {1.3, 1.11, 1.3, 0.55, 0.7, 0.85};
  float VCurrMin[kNACurr] = { 0.25, 0.2, 0.0, 0.1, 0.00};
  float VCurrMax[kNACurr] = { 0.55, 0.4, 0.2, 0.2, 0.04};
  float ACurrMin[kNACurr] = { 80, 50,  0, 40, 10};
  float ACurrMax[kNACurr] = {110, 80, 50, 60, 50};

  int nHists = 0;
  int j = 0;
  char id[50];

  for (j = 0; j<kNVRef, j++) {

  } 
  for (j = 0; j<kNACurr, j++) {
    sprintf(id, "h%d", nHists);
    h[nHists] = new TH1F(id, VRefText[j], VRefMin[j], VMax[j]);
    AddHist(nHists, VCurrText[j], VCurrMin[j], VCurrMax[j]);
    nHists++;
    AddHist(nHists, ACurrText[j], ACurrMin[j], ACurrMax[j]);
    nHists++;
  } 


  float Gain20mVMin = ;
  float Gain20mVMax = ;
  float Gain30mVMin = ;
  float Gain30mVMax = ;

  float RT20mVMin = ;
  float RT30mVMin = ;
  float RT20mVMax = ;
  float RT30mVMax = ;

  float Ped20mVMin = ;
  float Ped30mVMin = ;
  float Ped20mVMax = ;
  float Ped30mVMax = ;

  float Rms20mVMin = ;
  float Rms30mVMin = ;
  float Rms20mVMax = ;
  float Rms30mVMax = ;

  float PedAna20mVMin = ;
  float PedAna30mVMin = ;
  float PedAna20mVMax = ;
  float PedAna30mVMax = ;
  float RmsAna20mVMin = ;
  float RmsAna30mVMin = ;
  float RmsAna20mVMax = ;
  float RmsAna30mVMax = ;

  float X20mVevenMin = ;
  float X20mVoddMin = ;
  float X30mVevenMin = ;
  float X30mVoddMin = ;

  float X20mVevenMax = ;
  float X20mVoddMax = ;
  float X30mVevenMax = ;
  float X30mVoddMax = ;

  float ClkMin = 95;
  float ClkMax = 135;

  TH1F *hV4_Clk = new TH1F("hV4_Clk", "Ring Oscillator", nbins, minClk, maxClk);

  TH1F *hV3C[kNV];
  TH1F *hV4C[kNV];

  TH1F *hV3[kNV];
  TH1F *hV4[kNV];

  char id[50], title[50];
  for (int iC=0; iC<kNC; iC++) {
    sprintf(id, "hV3%dC", iC);
    sprintf(title, "%s", curText[iC]);
    hV3C[iC] = new TH1F(id, title, nbins, curMin[iC], curMax[iC]);
    sprintf(id, "hV4%dC", iC);
    hV4C[iC] = new TH1F(id, title, nbins, curMin[iC], curMax[iC]);
  }

  for (int iV=0; iV<kNV; iV++) {
    sprintf(id, "hV3%d", iV);
    sprintf(title, "%s", voltText[iV]);
    hV3[iV] = new TH1F(id, title, nbins, voltMin[iV], voltMax[iV]);
    sprintf(id, "hV4%d", iV);
    hV4[iV] = new TH1F(id, title, nbins, voltMin[iV], voltMax[iV]);
  }

  Long64_t nbytes = 0;
  int ichan = 0;
  for (Long64_t i=0; i<nentries;i++) {
    nbytes += tree->GetEntry(i);

    TString s(version);
    if ( s.Contains("V3") ) {
      if ( (fAC>0 && fAC<1) &&
	   (fDC>0 && fDC<1) ) {
	hV3C[0]->Fill( fAC ); 
	hV3C[1]->Fill( fDC );
      }  
      if ( (iV770>100 && iV770<1000) &&
	   (iV450>100 && iV450<1000) &&
	   (iV620>100 && iV620<1000) ) {
	hV3[0]->Fill( iV770 ); 
	hV3[1]->Fill( iV450 );
	hV3[2]->Fill( iV770 - iV450);
	hV3[3]->Fill( iV770 - iV620);
      }  
      if (iClk>50) hV3_Clk->Fill( iClk ); 
    }
    else if ( s.Contains("V4") ) {
      if ( (fAC>0 && fAC<1) &&
	   (fDC>0 && fDC<1) ) {
	hV4C[0]->Fill( fAC ); 
	hV4C[1]->Fill( fDC );
      }  
      if ( (iV770>100 && iV770<1000) &&
	   (iV450>100 && iV450<1000) &&
	   (iV620>100 && iV620<1000) ) {
	hV4[0]->Fill( iV770 ); 
	hV4[1]->Fill( iV450 );
	hV4[2]->Fill( iV770 - iV450);
	hV4[3]->Fill( iV770 - iV620);
      }  
      if (iClk>50) hV4_Clk->Fill( iClk ); 
    }
  }

  c1->cd(1);
  //  c1->SetLogy(1);

  // first the clock plots
  hV3_Clk->SetLineColor(2);
  hV4_Clk->SetLineColor(4);

  gStyle->SetOptStat(0);

  hV3_Clk->Draw();
  hV4_Clk->Draw("SAME");

  char rmsinfo[20];
  TLegend* legend = new TLegend(0.6,0.6,0.9,0.9);
  legend->SetHeader("SAMPA - Clk");
  sprintf(rmsinfo,"V3 : #mu = %3.2f, #sigma = %3.2f", hV3_Clk->GetMean(), hV3_Clk->GetRMS() );
  legend->AddEntry(hV3_Clk,rmsinfo,"l");
  sprintf(rmsinfo,"V4 : #mu = %3.2f, #sigma = %3.2f", hV4_Clk->GetMean(), hV4_Clk->GetRMS() );
  legend->AddEntry(hV4_Clk,rmsinfo,"l");
  legend->Draw();

  c1->Modified();
  c1->SaveAs("V34_Clk.pdf");

  // Next the current plots
  TLegend* legendC[kNC];
  for (int iC=0; iC<kNC; iC++) {
    c1->cd(1);
    //  c1->SetLogy(1);
    hV3C[iC]->SetLineColor(2);
    hV4C[iC]->SetLineColor(4);
    
    if ( hV4C[iC]->GetMaximum() > hV4C[iC]->GetMaximum() ) {
      hV4C[iC]->Draw();
      hV3C[iC]->Draw("SAME");
    }
    else {
      hV3C[iC]->Draw();
      hV4C[iC]->Draw("SAME");
    }

    legendC[iC] = new TLegend(0.6,0.6,0.9,0.9);
    legendC[iC]->SetHeader("SAMPA - Currents");
    sprintf(rmsinfo,"V3 : #mu = %3.2f, #sigma = %3.2f", hV3C[iC]->GetMean(), hV3C[iC]->GetRMS() );
    legendC[iC]->AddEntry(hV3C[iC],rmsinfo,"l");
    sprintf(rmsinfo,"V4 : #mu = %3.2f, #sigma = %3.2f", hV4C[iC]->GetMean(), hV4C[iC]->GetRMS() );
    legendC[iC]->AddEntry(hV4C[iC],rmsinfo,"l");
    legendC[iC]->Draw();

    c1->Modified();
    sprintf(title, "V34_current%d.pdf", iC);
    c1->SaveAs(title);
  }

  // Next the voltage plots
  TLegend* legendV[kNV];
  for (int iV=0; iV<kNV; iV++) {
    c1->cd(1);
    //  c1->SetLogy(1);
    hV3[iV]->SetLineColor(2);
    hV4[iV]->SetLineColor(4);
    
    hV3[iV]->Draw();
    hV4[iV]->Draw("SAME");
    
    legendV[iV] = new TLegend(0.6,0.6,0.9,0.9);
    legendV[iV]->SetHeader("SAMPA - Bandgap");
    sprintf(rmsinfo,"V3 : #mu = %3.2f, #sigma = %3.2f", hV3[iV]->GetMean(), hV3[iV]->GetRMS() );
    legendV[iV]->AddEntry(hV3[iV],rmsinfo,"l");
    sprintf(rmsinfo,"V4 : #mu = %3.2f, #sigma = %3.2f", hV4[iV]->GetMean(), hV4[iV]->GetRMS() );
    legendV[iV]->AddEntry(hV4[iV],rmsinfo,"l");
    legendV[iV]->Draw();

    c1->Modified();
    sprintf(title, "pdf/V34_voltage%d.pdf", iV);
    c1->SaveAs(title);
  }
  */
}
