/*use this code just to get the noise and baseline for 32 ch
This codes has three important functions:
[1] anl_tree ()
This function reads root tree packet-by-packet and contains pulse fitting part
[2] hist_anal()
Here, final plots are made
[3] SaveHST()
Using this one can save histograms in root file

//To run the code
(1) go to root terminal using command: 

root -l

(2) load the code: 

.L Noise-anal_sampa_dataCh_fit.cpp

(3) call the function and give Path to the root file like for e.g.

anl_tree("SAMPA_test.root")

(4) to save the analysis in terms of root histograms call function SaveHST()
 and give the filename you want as follows;

 SaveHST("results.root") 
 
 if you have more root files (e.g. result_1.root and result_2.root) 
 of same test settings for analysis then; 
 
 SaveHST("results_*.root")   

(5) To get the final results call funtion hist_anal(); 
this will generate 2-D histograms and data file, for e.g. noise vs ch number and 
respective data points in .txt file

Remember do not quit root unless you are done with analysis (all above steps)
If you find any mistake or wish to give comment/suggetion write to:
gatambav@cern.ch or gta023@uib.no
*/
//---------------------------------------------------------------------------------------------
#include "TH1F.h"
#include "TH2F.h"
#include "TMath.h"
#include "TF1.h"
#include "TLegend.h"
#include "TCanvas.h"
//
#include "TTree.h"
#include "TFile.h"
#include "TChain.h"
#include "TStyle.h"
#include <string>
#include <cmath>
#include <iostream>
//
#define Nch 32
const Int_t kWindow = 10; // +- kWindow bins around max for fitting, and mean calculation
//------------------------------
//DS gROOT->Reset();
TFile *inFile;
TTree *tree;
//
TH1F* h_Trace[Nch];
TH1F *h_noise[Nch]; 
TH1F *h_noise_fit[Nch]; 
//
TH1F *h_ChID;
TH1F *h_leng;
TH2F *h_len_ch;

//==============================================================================
void init_hst(){
    char tstr[100];
    for(int i=0;i<Nch;i++){
      sprintf(tstr,"h_Trace_%d",i); 
      h_Trace[i] = new TH1F(tstr,tstr,1004,-0.5,1003.5);
      sprintf(tstr,"h_noise_%d",i); 
      h_noise[i] = new TH1F(tstr,tstr,1024,-0.5,1023.5);
      sprintf(tstr,"h_noise_fit_%d",i); 
      h_noise_fit[i] = new TH1F(tstr,tstr,1024,-0.5,1023.5);
    }
    h_ChID = new TH1F("h_ChID", "h_ChID", 33,-0.5,32.5);
    h_leng = new TH1F("h_leng", "h_leng", 9,997.5,1006.5);   
    h_len_ch = new TH2F("h_len_ch", "h_len_ch", 9,997.5,1006.5,33,-0.5,32.5);
}

//===========================Read root Tree===================================================
void anl_tree(const char *fname="SAMPA_test.root"){
  UShort_t samples[1024]; // 2 bytes 
  UShort_t len;
  UChar_t hadd;
  UChar_t ch;
  // init Tree for single file
  inFile = new TFile(fname);
  tree = (TTree*)inFile->Get("DAQ_Events");
  //
  if(!tree){
    std::cerr << "Tree DAQ_Events was not found\n";
  return;
  }
  //
  tree->SetBranchAddress("samples",&samples);
  tree->SetBranchAddress("len",&len);
  tree->SetBranchAddress("hadd",&hadd);
  tree->SetBranchAddress("chanadd",&ch);
  //----------------------------------------------------
  init_hst();
  Int_t nentries = (Int_t)tree->GetEntries();
  std::cout <<"#events: "<< nentries << "\n";
  h_len_ch->SetMarkerStyle(2);
  int good_event = 0;
  //Read entries in the tree
  int i_interval = nentries / 10;
  for (Int_t i = 0; i < nentries; i++){
    tree->GetEntry(i);
    if (i%i_interval == 0) printf("entry %i, chip %d, ch %i\n", i, hadd, ch);

    /* 20170126 - disabled bad sample check */
    // 20171206 enabled again...
    bool bad_sample = false;
    for (Int_t l = 2; l < len; ++l)
      if (samples[l] > 200) { //signature of bad entry
	bad_sample = true;
	break;
      }
    if (bad_sample)
      continue;

    h_ChID->Fill(ch);
    h_leng->Fill(len);
    h_len_ch->Fill(len,ch);
    //---------------Start of Pulse analysis (Pulse and Noise)---------
    for(Int_t l = 2; l < len; l++){
      h_Trace[ch]->SetBinContent(l-2,samples[l]);
      h_noise[ch]->Fill(samples[l]);
      h_noise_fit[ch]->Fill(samples[l]);
    }
  } //End-of-TreeRead
} // End-of-analysis
//=======fit-gaussian/plot/saveData============================================
void hist_anal(){
  //
  FILE *f = fopen("ns_vs_chNo.txt", "w");
  if (f == NULL){ 
    printf("Error opening file!\n");
    exit(1);
  }
  //--------------------
  TH1F *h_ns_ChNo   = new TH1F("h_ns_ChNo","ns_vs_chNo",32,-0.5,31.5);
  TH1F *h_bl_ChNo   = new TH1F("h_bl_ChNo","baseline_vs_chNo",32,-0.5,31.5);

  h_ns_ChNo->SetMinimum(0);
  //  h_ns_ChNo->SetMaximum(2);

  h_bl_ChNo->SetMinimum(0);
  h_bl_ChNo->SetMaximum(200);

 fprintf(f,"ch hNent  hMean hRMS   gMean gSigma\n");

  for(int i = 0; i < Nch; i++){

    //noise_vs_ch_Number
    int hist_nent = (int)h_noise[i]->GetEntries();

    // restrict range for mean and fit calculation DS
    Int_t nbins = h_noise[i]->GetNbinsX();
    Int_t imaxBin = h_noise[i]->GetMaximumBin();
    Int_t iBinLow = imaxBin - kWindow;    
    Int_t iBinHigh = imaxBin + kWindow;
    // check that not out of range
    if (iBinLow<1) iBinLow = 1;
    if (iBinLow>nbins) iBinLow = nbins;
    h_noise[i]->GetXaxis()->SetRange(iBinLow, iBinHigh);

    //
    double hist_mean = h_noise[i]->GetMean();
    double hist_RMS = h_noise[i]->GetRMS();
    //printf("%i %.2f %.4f\n", i, hist_mean, hist_RMS);

    // 20170120: unrestrict range to show all bins with entries
    int ibin=1; 
    while ( (h_noise[i]->GetBinContent(ibin) == 0) && (ibin<nbins) ) {
      ibin++;
    }
    iBinLow = ibin;

    ibin = nbins; 
    while ( (h_noise[i]->GetBinContent(ibin) == 0) && (ibin>1) ) {
      ibin--;
    }
    iBinHigh = ibin;

    // if not out of range, add extra bin on each side
    if (iBinLow>1) iBinLow--;
    if (iBinHigh<nbins) iBinHigh++;
    h_noise[i]->GetXaxis()->SetRange(iBinLow, iBinHigh);

    if(hist_mean > 0.){
      h_noise_fit[i]->Fit("gaus","Q"," ",hist_mean-kWindow,hist_mean+kWindow);
      TF1 *fit_amp = h_noise_fit[i]->GetFunction("gaus");
      float mean = fit_amp->GetParameter(1);
      float sigma = fit_amp->GetParameter(2);
      fprintf(f,"%02i %06i %.2f %.4f %.2f %.4f\n", i, hist_nent, hist_mean, hist_RMS, mean, sigma);
      h_ns_ChNo->SetBinContent(i+1,sigma);
      h_bl_ChNo->SetBinContent(i+1,mean);
    }
    else {
      fprintf(f,"%02i %i %.2f %.4f %.2f %.4f\n", i, hist_nent, hist_mean, hist_RMS, -999.9, -999.9);
    }
  }
  fclose(f);
  //
  TCanvas *c11 = new TCanvas("c11","SAMPA_plots",600,400);
  c11->Divide(1,2);
  //

  h_bl_ChNo->SetMarkerStyle(4);
  h_ns_ChNo->SetMarkerStyle(8);

  //
  gStyle->SetOptStat(0);
  c11->cd(1); h_ns_ChNo->Draw("P");
  c11->cd(2); h_bl_ChNo->Draw("P");
  c11->SaveAs("SAMPA_plots.pdf");

  TCanvas *c2 = new TCanvas("c2","SAMPA_noise");
  c2->cd();
  h_ns_ChNo->Draw("P");
  h_ns_ChNo->GetXaxis()->SetTitle("Chan #");
  h_ns_ChNo->GetYaxis()->SetTitle("Sigma (ADC)");
  c2->SaveAs("SAMPA_noise.pdf");

}
//===================save-histograms-to-file========================
void SaveHST(const char *fname="results.root"){
  TFile file(fname,"RECREATE");
  file.cd();
  for(int i = 0; i < Nch; i++){
    h_noise[i]->Write();
    h_Trace[i]->Write();
  }
  h_ChID->Write();
  h_leng->Write();
  h_len_ch->Write();
  file.Close();
}
