#!/usr/bin/env perl

use strict;
use List::MoreUtils qw(uniq);
use Data::Dumper qw(Dumper);

# global hardcoded constants
#test
my $minSync = 11; # all links should be synched
my $minJtagWalk = 24;
my $maxMemErr = 0;
my $minClock = 100;
my $maxClock = 130;

my $minPedVal = 30;
my $maxPedVal = 105;
my $minPedOK = 64;
my $maxPedNotOK = 0;

my $minDFT = 2;

#ana
my $minChan = 32;
my $maxProblem = 0;

my $topdir = "logs";
my $outputFilename = "$topdir/summary.csv";
my $writeLog = 0; # set to 1 to write to logs
# end hardcoded constants

my $chipnum = 0;
if($#ARGV + 1 > 0)
{
    $chipnum = shift @ARGV;
}
else
{
    print "Please provide chipnumber argument\n";
    exit;
}

print "Checking logs for chip ${chipnum}:\n";

my $chipstring = "-chip${chipnum}.log";
open(my $fh, "-|", "ls -r $topdir/*${chipstring}");

my $teststring = "test-";
my $syncstring = "Synced: 1";
my $unsyncstring = "Synced: 0";
my $jtagClkErrstring = "JTAG\_clock";
my $jtagPinErrstring = "Error: Wrong value on pin";
my $walkstring = "Walking zero"; #jtag
my $memorystring = "Memory errors";
my $clockstring = "Clock speed";
my $baselinestring = "Baseline test";
my $colonstring = "\;";
my $syncCount = 0;
my $unsyncCount = 0;
my $jtagClkErrCount = 0;
my $jtagPinErrCount = 0;
my $walkCount = 0;
my $memErrCount = 0;
my $ringOscVal = 0;
my $baselineCount = 0;
my $baselineErrCount = 0;
my $dftCount = 0;

# debug print
my $jtagErrstring = "JTAG Errors:\n";
my $clockCheck = `grep Clock $topdir/test${chipstring}`;
print "$clockCheck\n";

my $testDFTstring = "testDFT";
my $dftstring = "Fantastisk, No errors";

my $anastring = "ana";
my $countstring = "count";
my $allstring = "all";
my $problemstring = "problem";
my $channelCount = 0;
my $problemCount = 0;

my $filecount = 0;
my $line = "";
my $infostring = "";
my $splitstring = "\;";


while(my $filename = <$fh>) 
{
    chomp($filename);
#    print "filename $filename\n";

    if ($filename =~ /$teststring/) {
	print "\n test file\n";
	open( my $tfh, '<', $filename ) or die "Can't open $filename: $!";
	while ( $line = <$tfh> ) {
	    if ( $line =~ /$syncstring/ ) { $syncCount++; }
	    if ( $line =~ /$unsyncstring/ ) { $unsyncCount++; }
	    elsif ( $line =~ /$jtagClkErrstring/ ) { 
		$jtagClkErrCount++; 
#		$jtagErrstring .= "$line";
	    }
	    elsif ( $line =~ /$jtagPinErrstring/ ) { 
		$jtagPinErrCount++; 
		$jtagErrstring .= "$line";
	    }
	    elsif ( $line =~ /$walkstring/ ) { $walkCount++; }
	    elsif ( $line =~ /$memorystring/ ) { 
		my @memarr = split / /, $line;
		$memErrCount = $memarr[2] + 0; 
	    }
	    elsif ( $line =~ /$clockstring/ ) { 
		my @clockarr = split / /, $line;
		$ringOscVal = $clockarr[2] + 0; 
	    }
	    if ( $line =~ /$baselinestring/ ) {
# line after next has the actual values
		$line = <$tfh>; 
		$line = <$tfh>; 
		if ($line =~ /$colonstring/ ) {}
		else { # something extra came in, read again
		    $line = <$tfh>; 
		}
#		print "baseline $line\n";
		chop $line;
		my @baselinearr = split /$splitstring/, $line;

		foreach my $bl (@baselinearr) {
#		    print "bl $baselineCount $bl \n";
		    if ( ($bl < $minPedVal) || ($bl > $maxPedVal) ) {
			print "out-of-bounds bl $baselineCount : val $bl \n";
			$baselineErrCount++;
		    }
		    $baselineCount++;
		}
	    }
	}
	close $tfh;
	print "syncCount $syncCount\n";
	print "unsyncCount $unsyncCount\n";
	print "jtagClkErrCount $jtagClkErrCount\n";
	print "jtagPinErrCount $jtagPinErrCount\n";
	print "walkCount $walkCount\n";
	print "memErrCount $memErrCount\n";
	print "ringOscVal $ringOscVal\n";
	print "baselineCount $baselineCount\n";
	print "baselineErrCount $baselineErrCount\n";
    }    

    elsif ($filename =~ /$anastring/) {
	print "\n ana file\n";
# DS counter for missing channels; string with missing channels
	open( my $afh, '<', $filename ) or die "Can't open $filename: $!";
	while ( my $line = <$afh> ) {
	    if ( $line =~ /$countstring/ ) {
#		print $line;
		my $infostring = $line;
		$line = <$afh>; 
		chomp $line;
		if ($infostring =~ /$allstring/ ) {
		    $channelCount = $line + 0;
#		    print "all $line\n";
		}
		elsif ($infostring =~ /$problemstring/ ) {
		    $problemCount = $line + 0;
		    print "problem $line\n";
		}
	    }
	}
	print "channelCount $channelCount\n";
	print "problemCount $problemCount\n";
	if ($problemCount > 0) {
	    my $nlines = $problemCount + 2;
	    my $anaErrstring = `tail -n $nlines $topdir/ana${chipstring}`;
	    print "Problem channels:\n $anaErrstring\n";
	}
	close $afh;
    }    

    elsif ($filename =~ /$testDFTstring/) {
	print "\n test DFT file\n";
	my $dftDone = 0;
	my $ndftCheck = 0;
	while ( ($dftDone == 0) && ($ndftCheck<60) ) {
	    sleep 1;
	    $dftDone = `tail -2 $filename | grep DONE | wc -l`;
	    $dftDone = $dftDone + 0;
	    $ndftCheck++;
	    print "DFT check $ndftCheck : dftDone $dftDone\n";
	}

	open( my $dfh, '<', $filename ) or die "Can't open $filename: $!";
	while ( $line = <$dfh> ) {
	    if ( $line =~ /$dftstring/ ) { $dftCount++; }
	}
	close $dfh;
	print "dftCount $dftCount\n";
    }    

    $filecount++;
}

#print "found $filecount files\n";

		
#test
my $syncOK = 1;
my $JtagOK = 1;
my $memOK = 1;
my $clockOK = 1;
my $PedOK = 1;
my $DFTOK = 1;
if ( ($syncCount < $minSync) || ($unsyncCount > 0) ) { $syncOK = 0; }
#if ( ($walkCount < $minJtagWalk) || ($jtagClkErrCount > 0) || ($jtagPinErrCount > 0) )  { $JtagOK = 0; }
if ( ($walkCount < $minJtagWalk) || ($jtagPinErrCount > 0) )  { $JtagOK = 0; }
if ($memErrCount > $maxMemErr) { $memOK = 0; }
if ( ($ringOscVal < $minClock) || ($ringOscVal > $maxClock) ) { $clockOK = 0; }
if ( ($baselineCount < $minPedOK) || ($baselineErrCount > $maxPedNotOK) ) { $PedOK = 0; }
if ($dftCount < $minDFT) { $DFTOK = 0; }

#ana
my $TPOK = 1;
if ( ($channelCount < $minChan) || ($problemCount > $maxProblem) ) { $TPOK = 0; }
#DS report missing channels

print "\nSummary: Chip $chipnum\n";
print "SyncOK $syncOK\n";
print "JtagOK $JtagOK\n";
if ($JtagOK != 1) {
    print "$jtagErrstring \n";
}
print "MemOK $memOK\n";
print "ClockOK $clockOK\n";
printf("RingOscVal %3d\n", ($ringOscVal+0.5));
print "PedOK $PedOK\n";
print "DFTOK $DFTOK\n";
print "TPOK $TPOK\n";

if ( ($syncOK == 1) && ($JtagOK == 1) && ($memOK == 1) && ($clockOK == 1)
     && ($PedOK == 1) && ($DFTOK == 1) && ($TPOK == 1) ) {
    print "==> GOOD ! \n";
}
else {
    print "==> BAD ! \n";
}

# header for OUTP:
# s=sync, j=jtag,m=mem, c=clk, p=ped, d=dft, t=tp
# chip, s, j, m, c, p, d,t
# -------------------------
if ($writeLog == 1) {
    open(OUTP, ">>$outputFilename");
    print OUTP "$chipnum, $syncOK, $JtagOK, $memOK, $clockOK, $PedOK, $DFTOK, $TPOK\n";
    close OUTP;
}

