#include "TH1F.h"
#include "TProfile.h"
#include "TMath.h"
#include "TF1.h"
#include "TLegend.h"
#include "TCanvas.h"
 //
#include "TTree.h"
#include "TFile.h"
#include "TChain.h"
#include "TStyle.h"
#include <string>
#include <cmath>
#include <iostream>
#include <vector>
#include <algorithm>
 //

Double_t Fit_Function(Double_t* Coordinates, Double_t* Parameters){
  Double_t N = 4; //constant 
  Double_t Tau = Parameters[0]; //Peaking time
  Double_t time_stamp = Parameters[1];  //start-time
  Double_t bl = Parameters[2];  //baseline
  Double_t A = Parameters[3];   //peak
  Double_t x = Coordinates[0];
  if ((x - time_stamp) < 0)
    return bl;
  else
    return bl + A * TMath::Exp(-N * (x - time_stamp) / Tau) * TMath::Power((x - time_stamp) / Tau, N);
 }

void staircaseWithTree(const char *listname="staircaseWithTreelist.txt", 
		       const char *outname="staircaseWithTree.root")
{
  gStyle->SetOptStat(0);
  //  gStyle->SetOptFit(0);
  gStyle->SetOptFit(0111);
  gStyle->SetStatX(0.5); 

  const int debug = 1;
  //  const int debug = 3; //DS test
  //const bool regeneratePlots = false;
  const bool regeneratePlots = true;

  const int zoomWindow=5; // plot zoom in on peak
  //  const double kScaleFactorADC = 2.15; // for amp, from ADC to mV
  const double kScaleFactorADC = 1; // we do this conversion in fitStudy.C instead

  // chip 1: 1 pF (effective cap seems more like 1.172; tuned with reference chip), with 11 and 39 Ohm; fewer steps for V3 
  const int kNSteps = 7;
  const int kmVSteps[kNSteps] = { 350, 310, 270, 230, 160, 80, 10 };
  const double kScaleFactor = 11.0/(11.0 + 39.0) * 1.172; // multiply pulsegenerator mV with 

  /*
  const int kNSteps = 13;
  // chip 2: 1 pF (effective cap seems more like 1.172; tuned with reference chip), with 11 and 39 Ohm 
  const int kmVSteps[kNSteps] = { 360, 340, 320, 300, 280, 260, 240, 220, 180, 140, 100, 60, 20 };
  const double kScaleFactor = 11.0/(11.0 + 39.0) * 1.172; // multiply pulsegenerator mV with this to get fC
  */
  /*
  // chip 3: 10 pF, with 2.35 and 47 Ohm
  const int kmVSteps[kNSteps] = { 180, 170, 160, 150, 140,
				  130, 120, 110, 90, 70,
				  50, 30, 10 };
  const double kScaleFactor = 2.35/(2.35+47.0) * 10.0; // multiply pulsegenerator mV with this to get fC
  */
  const int kMaxPeaks = 20; // should really only be kNSteps intervals but use a lot of extras to avoid crashes 
  const int kLimit = 1;

  const double kPrecisionfC = 1; // there is some uncertainty on the fC scale

  const int kNChan = 32;
  
  char infostr[100];
 
  // variables common for all channels
  int iChip = 0;
  int iSig = 0;
  int iVref = 0;

  // fit variables
  int iChan = 0;
  float fP0 = 0;
  float fP0Err = 0;
  float fP1 = 0;
  float fP1Err = 0;
  float fChi2 = 0;
  int iNDF = 0;

  TFile *fout = new TFile(outname, "RECREATE");
  TTree* tree = new TTree("tree", "fit info");
 
  tree->Branch("iChip",&iChip,"iChip/I");
  tree->Branch("iChan",&iChan,"iChan/I");
  tree->Branch("fP0",&fP0,"fP0/F");
  tree->Branch("fP0Err",&fP0Err,"fP0Err/F");
  tree->Branch("fP1",&fP1,"fP1/F");
  tree->Branch("fP1Err",&fP1Err,"fP1Err/F");
  tree->Branch("fChi2",&fChi2,"fChi2/F");
  tree->Branch("iNDF",&iNDF,"iNDF/I");

  ifstream input;
  input.open(listname);
  char filename[200];
  char outdir[50];
  char command[100];

  int nfiles = 0;
  while (1) {
    input >> filename;
    if (! input.good()) break;
    nfiles++;
    
    TString s(filename);
    int firstR = s.First(".");  // start of ".root"
    int length = s.Length();
    int nChar = 3 - (58-length); // string length 58 for 3 digit numbers
    if (debug>0) {
      cout << " firstR " << firstR
           << " length " << length
           << " nChar " << nChar
           << endl;
    }

    TString desc ( s(firstR-nChar, nChar ) ); 
    if (debug>0) cout << " desc " << desc << endl;
    
    iVref = 0; // internal
    if ( desc.Contains("ext") ) {
      iVref = 1;
    }
    
    TString chipstr ( desc(0, nChar) );
    if (debug>0) cout << " chipStr " << chipstr << endl;
    iChip = chipstr.Atoi();
    
    cout << " iChip " << iChip
	 << " iSig " <<  iSig
	 << " iVref " << iVref << endl;

    if (regeneratePlots) {
      char chipVer[10];
      if (iChip<2000) {
	sprintf(chipVer, "V2");
      }
      else if (iChip>=2000 && iChip<6000) {
	sprintf(chipVer, "V3");
      }
      else if (iChip>=6000) {
	sprintf(chipVer, "V4");
      }

      sprintf(outdir, "png/calib/%s/%d", chipVer, iChip);

      sprintf(command, ".! mkdir -p %s", outdir);
      gROOT->ProcessLine(command);
      cout << " executing command " << command << endl;
    }

    TFile *f = new TFile(filename);
    cout << " reading file " << filename << endl;
    TTree *DAQ_Events = (TTree*) f->Get("DAQ_Events");

    //Declaration of leaves types
    ULong64_t       seconds;
    ULong64_t       useconds;
    UShort_t        len;
    UChar_t         chanadd;
    UShort_t        samples[2000]; // should really only be 1002, but we pad some extra length
     
    // Set branch addresses.
    DAQ_Events->SetBranchAddress("seconds",&seconds);
    DAQ_Events->SetBranchAddress("useconds",&useconds);
    DAQ_Events->SetBranchAddress("len",&len);
    DAQ_Events->SetBranchAddress("chanadd",&chanadd);
    DAQ_Events->SetBranchAddress("samples",samples);
    
    //       To read only selected branches, Insert statements like:
    DAQ_Events->SetBranchStatus("*",0);  // disable all branches
    DAQ_Events->SetBranchStatus("seconds",1);  // activate branchname
    DAQ_Events->SetBranchStatus("useconds",1);  // activate branchname
    DAQ_Events->SetBranchStatus("len",1);  // activate branchname
    DAQ_Events->SetBranchStatus("chanadd",1);  // activate branchname
    DAQ_Events->SetBranchStatus("samples",1);  // activate branchname
    
    Long64_t nentries = DAQ_Events->GetEntries();
    cout << " nentries " << nentries << endl;
    if (debug>1) nentries = 2; // DS test

    Long64_t nbytes = 0;
    int i;
    
    // signal stuff
    int ichan = 0;

    // local var in loop
    int maxBin = 0;
    int maxVal = 0;
    int rangeMin = 0;
    int rangeMax = 0;
    double amp = 0;
    double base = 0;
    int a = 0;

    char hname[100];
    char hdesc[100];
    TProfile* hPeaks[32];
    for (ichan=0; ichan<32; ichan++) {
      sprintf(hname, "hPeaks_%d", ichan);
      sprintf(hdesc, "Peaks - Chan %d", ichan);
      hPeaks[ichan] = new TProfile(hname, hdesc, 
				   kNSteps+1, -0.5, kNSteps + 0.5);
    }

    // event loop
    for (i=0; i<nentries;i++) {
      nbytes += DAQ_Events->GetEntry(i);
      if (len!=1002) continue; // skip events with strange lengths
      
      ichan = chanadd;
      if (ichan>=0 && ichan<32) {

	int nZeros = 0;
	int maxnZeros = 0;
	int firstAfterZeros = -999;
	int lastAfterZeros = -999;

	for(Int_t l = 2; l < len; l++) {
	  if (samples[l] == 0) {    nZeros++;      }
	  else {
	    if (nZeros > 10) {
	      lastAfterZeros = l-2;
	      if (firstAfterZeros==-999) { firstAfterZeros = l-2; }
	    }
	    if (nZeros > maxnZeros) {
	      maxnZeros = nZeros;
	    }
	    nZeros = 0;
	  }
	  if (debug>1) printf("sample %d adc %d nZerosInARow %d\n", l-2, samples[l], nZeros);
	}

	if (debug>1) {
	  cout << " firstAfterZeros " << firstAfterZeros << endl;
	  cout << " lastAfterZeros " << lastAfterZeros << endl;
	  cout << " maxnZeros " << maxnZeros << endl;
	}

	rangeMin = TMath::Max(firstAfterZeros, 1);
	rangeMax = TMath::Min(lastAfterZeros - maxnZeros, len - 2);
	maxVal = 0;
	maxBin = 0;
	for(Int_t l = rangeMin; l < rangeMax; l++) {
	  if (maxVal<samples[l]) {
	    maxVal = samples[l];
	    maxBin = l;
	  }
	}

	if (debug>1) {
	  cout << " rangeMin " << rangeMin << endl;
	  cout << " rangeMax " << rangeMax << endl;
	  cout << " maxBin " << maxBin << endl;
	}

	//--get baseline
	base = 0;                                                   
    	if(maxBin <= (len - 20 - 2)){ // Take baseline before pulse        
	  for(Int_t a = (maxBin - 20); a < (maxBin - 10); a++)
	    base += samples[a];
	}
	else{ // Take baseline after pulse (this should not happen)          
	  for(Int_t a = (maxBin + 10); a < (maxBin + 20); a++)
	    base += samples[a];
	}
	base /= 10.;
	if (debug>1) printf("baseline estimate %3.1f\n", base);
	
	// get peak info
	int maxBinPeak[kMaxPeaks] = { 0 };
	int maxValPeak[kMaxPeaks] = { 0 };

	int nPeaks = 0;
	int ibin = rangeMin;
	while (ibin < rangeMax) {
	  // a maximum should be bigger than the earlier samples and later samples          
	  // and the neighbor samples should also be bigger than their neighbors            
	  if ( ((samples[ibin-1]+kLimit) < samples[ibin]) &&
	       ((samples[ibin+1]+kLimit) < samples[ibin]) &&
	       ((samples[ibin-2]+kLimit) < samples[ibin-1]) &&
	       ((samples[ibin+2]+kLimit) < samples[ibin+1]) ) {
	    maxBinPeak[nPeaks] = ibin;
	    maxValPeak[nPeaks] = samples[ibin];
	    nPeaks++;
	  }
	  ibin++;
	}

	// if expected amount of peaks, fill TProfile
	if (debug>1) printf("nPeaks %d\n", nPeaks);
	if (nPeaks == kNSteps) {
	  // also check that spacing between all peaks is the same
	  int spacing = maxBinPeak[nPeaks-1] - maxBinPeak[nPeaks-2]; 
	  bool spacingOK = true;
	  int ipeak = 0;
	  for (ipeak=0; ipeak<nPeaks; ipeak++) {
	    if (ipeak < (nPeaks-1) ) {
	      if ( (maxBinPeak[ipeak+1] - maxBinPeak[ipeak]) != spacing) { 
		spacingOK = false; 
	      }
	    }
	    if (spacingOK) {
	      amp = maxValPeak[ipeak] - base;
	      hPeaks[ichan]->Fill(ipeak, amp);
	      if (debug>2) {
		printf("ipeak %d maxBin %d amp %3.0f\n",
		       ipeak, maxBinPeak[ipeak], amp);
	      }
	    }
	  } // peak loop

	  if (spacingOK) { // add a gauge of the baseline as the last entry
	    int baseBin = maxBinPeak[nPeaks-1] + spacing;
	    if (baseBin<1002) {
	      amp = samples[baseBin] - base;
	      hPeaks[ichan]->Fill(kNSteps, amp);
	      if (debug>2) {
		printf("baseBin %d amp %3.0f\n",
		       baseBin, amp);
	      }
	    }
	  }

	} // all peaks found

      } // iChan
      
    } // entries

    // go through all the maxVal and baseline info and fill graphs
    TGraphErrors *g1[kNChan];
    int nOK[kNChan] = {0};
    double rms = 0;

    for (ichan=0; ichan<32; ichan++) { 

      // make a graph for each channel
      g1[ichan] = new TGraphErrors(kNSteps + 2); 
      g1[ichan]->SetMarkerStyle(20);
      //   g1[ichan]->SetMarkerColor(ichan + 1);
      g1[ichan]->SetTitle("Max - Baseline");

      if (debug>0) { 
	cout << " ichan " << ichan << " : " 
	     << hPeaks[ichan]->GetEntries() << " values found" << endl;
      }

      if ( hPeaks[ichan]->GetEntries() > 0 ) {

	/*
	// add baseline info
	amp = hPeaks[ichan]->GetBinContent(kNSteps+1);
	rms = hPeaks[ichan]->GetBinError(kNSteps+1);
	g1[ichan]->SetPoint(nOK[ichan], 0, amp);
	g1[ichan]->SetPointError(nOK[ichan], kPrecisionfC, rms);
	if (debug>2) {
	  printf("ichan %d base amp %4.1f rms %4.1f\n",
		 ichan, amp, rms);
	}
	nOK[ichan]++;
	*/
	g1[ichan]->SetPoint(nOK[ichan], 0, 0);
	g1[ichan]->SetPointError(nOK[ichan], kPrecisionfC, 0.1);
	nOK[ichan]++;

	// figure out amp and rms for each step a.k.a. peak
	int ipeak = 0;
	for (ipeak=kNSteps-1; ipeak>=0; ipeak--) {	
	// add points to graph
	  double fC_estimate = kmVSteps[ipeak] * kScaleFactor;
	  amp = kScaleFactorADC * hPeaks[ichan]->GetBinContent(ipeak+1);
	  rms = kScaleFactorADC * (hPeaks[ichan]->GetBinError(ipeak+1) + 0.1); // smear a little 
	  if (debug>2) {
	    printf("ichan %d ipeak %d fC_estimate %4.1f amp %4.1f rms %4.1f\n",
		   ichan, ipeak, fC_estimate, amp, rms);
	  }
	  // add points to graph
	  g1[ichan]->SetPoint(nOK[ichan], fC_estimate, amp); 
	  g1[ichan]->SetPointError(nOK[ichan], kPrecisionfC, rms);
	  nOK[ichan]++;
	}
      }      // some fillings for this channel
    } // ichan loop

    // graphs are filled - now we do the fitting...

    // fit channel info, and store fit parameters (with errors) and chi2 values
    char name[50], description[100];
    TCanvas* c1 = new TCanvas("c1", "Amplitude vs Sig", 900, 600);
    TCanvas* c2 = new TCanvas("c2", "Amplitude vs Sig - Chip", 900, 600);

    TGraphErrors *gp0 = new TGraphErrors(kNChan);
    gp0->SetMarkerStyle(20);
    gp0->SetTitle("Fit Par 0");
    TGraphErrors *gp1 = new TGraphErrors(kNChan);
    gp1->SetMarkerStyle(20);
    gp1->SetTitle("Fit Par 1");

    gStyle->SetOptFit(0111);

    for (ichan=0; ichan<kNChan; ichan++) {
      if (debug>1) cout << " ichan " << ichan << " nOK " << nOK[ichan] << endl; 
      if (nOK[ichan] < 2) continue; // don't try to fit an empty plot
      TF1 *func1 = new TF1("func1","[0] + [1]*x", 1, 99); // skip 0 point
      func1->SetParameter(0, 0);
      func1->SetParameter(1, 9.5);

      c1->cd();
      g1[ichan]->Fit("func1","RQ");

      if (regeneratePlots) {
	g1[ichan]->Draw("A*");
	sprintf(description, "Amplitude vs Sig for chip %d chan %d", iChip, ichan);
	g1[ichan]->GetXaxis()->SetTitle("signal (fC)");
	g1[ichan]->GetYaxis()->SetTitle("amplitude (ADC)");
	g1[ichan]->SetTitle(description);
	c1->Modified();
	sprintf(name, "%s/amp_sig_chip_%d_chan_%02d.png", outdir, iChip, ichan);
	c1->SaveAs(name);
	
	cout << " ichip " << iChip << " ichan " << ichan 
	     << " fit par1 " << func1->GetParameter(1)
	     << " error " << func1->GetParError(1) << endl;
      }

      iChan = ichan;
      fP0 = func1->GetParameter(0);
      fP0Err = func1->GetParError(0);
      fP1 = func1->GetParameter(1);
      fP1Err = func1->GetParError(1);
      fChi2 = func1->GetChisquare();
      iNDF = func1->GetNDF();
      tree->Fill();

      gp0->SetPoint(ichan, ichan, func1->GetParameter(0));
      gp0->SetPointError(ichan, 0, func1->GetParError(0));
      gp1->SetPoint(ichan, ichan, func1->GetParameter(1));
      gp1->SetPointError(ichan, 0, func1->GetParError(1));

      if (func1) delete func1;
    } // ichan

    gStyle->SetOptFit(0);

    if (regeneratePlots) {
      // plot fit info for all channels in chip
      c2->cd();
      sprintf(description, "Fit Par 0 : Chip %d", iChip);
      gp0->SetTitle(description);
      gp0->Fit("pol0","Q");
      gp0->Draw("A*");
      gp0->GetXaxis()->SetTitle("Chan #");
      gp0->GetYaxis()->SetTitle("Fit Par0");
      c2->Modified();
      sprintf(name, "%s/amp_sig_chip_%d_fitpar0.png", outdir, iChip);
      c2->SaveAs(name);
      
      c2->cd();
      sprintf(description, "Fit Par 1 : Chip %d", iChip);
      gp1->SetTitle(description);
      gp1->Fit("pol0","");
      gp1->Draw("A*");
      gp1->GetXaxis()->SetTitle("Chan #");
      gp1->GetYaxis()->SetTitle("Fit Par1");
      c2->Modified();
      sprintf(name, "%s/amp_sig_chip_%d_fitpar1.png", outdir, iChip);
      c2->SaveAs(name);
    }

    // clean up
    for (ichan=0; ichan<kNChan; ichan++) {
      if (g1[ichan]) delete g1[ichan];
    }
    if (gp0) delete gp0;
    if (gp1) delete gp1;

    if (f) delete f;
  }
  cout << " nfiles " << nfiles << endl;

  fout->Write();
}
