#!/bin/bash
if [ -z ${1+x} ]; then
    echo "at least one argument needed: chipnum";
    exit
fi

if [ -z ${2+x} ]; then
    echo "second argument (station) not supplied; assuming station 1";
    station=1
else
    station=$2
fi
mask=$(printf "%#x\n" ${station})
chip=$1
echo chip ${chip} mask ${mask} station ${station} 

today=`date +%Y%m%d` #YearMonthDay
outdir="/home/tpc/robot/${today}"
hms=`date +%H%M%S` #HourMinSec
prefix="${hms}_sampa"

chipstr=$(printf "%#06d" ${chip})
dir="${outdir}/${prefix}${chipstr}"
echo dir ${dir}

cd ~/sampa-test-stand/script
time run.py -m ${mask} --nr ${chip} --output-dir $outdir --nr-prefix ${prefix} --init --logname init.log
time run.py -m ${mask} --nr ${chip} --output-dir $outdir --nr-prefix ${prefix} --test-dc --logname dc0.log
time run.py -m ${mask} --nr ${chip} --output-dir $outdir --nr-prefix ${prefix} --test-readall --logname run.log
time run.py -m ${mask} --nr ${chip} --output-dir $outdir --nr-prefix ${prefix} --test-dc --logname dc.log
# maybe more tests to be added in "misc" for jtag etc?
time run.py -m ${mask} --nr ${chip} --output-dir $outdir --nr-prefix ${prefix} --test-misc --logname misc.log
# do DFT last...
time run.py -m ${mask} --nr ${chip} --output-dir $outdir --nr-prefix ${prefix} --test-dft --logname dft.log
# power-off when done for robot
time run.py -m ${mask} --nr ${chip} --output-dir $outdir --nr-prefix ${prefix} --off --logname off.log

perl checklog.pl ${dir} ${station} > ../logs/testchip${chip}.log

# get return code for robot
RETCODE=`grep Return ../logs/testchip${chip}.log | awk '{print $NF}'`
# store checklog summary in the exe dir also
cp ../logs/testchip${chip}.log ${dir}/.
cd -
exit ${RETCODE}

