# Bus Pirate

Bus Pirate adapted for programming of GBTx on TPC FEC

## Usage Examples
- Write configuration to GBTx at I2c address 0xa
```
./bpgbtx.py 0xa --write-cfg <FILENAME>
```

- Dump configuration to GBTx at I2c address 0xf to stdout or file
```
./bpgbtx.py 0xf --dump-cfg
./bpgbtx.py 0xf --dump-cfg <FILENAME>
```

- Specify alternative USB device to use, e.g. Bus Pirate on ttyUSB9
```
./bpgbtx.py --dev /dev/ttyUSB --write-cfg <FILENAME>
```

- Powercycle GBTx
```
  ./bpgbtx.py --powercycle
```

- Scan I2C for responding devices
```
./bpgbtx.py --i2c-scan
```

- Display help
```
./bpgbtx.py --help
./bpgbtx.py -h
```


## Setup
### Pin wiring
```
 Bus Pirate          FEC (GBTx0: J10, GBTx1: J22)
--------------      ----------------
 Pin 7 (purple) -->  Pin 7 (SCL)
 Pin 8 (gray)   -->  Pin 5 (SCA)
 Pin 5 (green)  -->  Pin 1 (1v5)
 Pin 6 (blue)   -->  Pin 6 (1v5_en)
 Pin 1 (brown)  -->  Pin 3 (GND)
```


### Notes

#### Required Packages
```
python3
pip3 
sudo pip3 install pyserial
```

#### System
udev rules for Bus Pirate
```bash
cat /etc/udev/rules.d/99-buspirate.rules
KERNEL=="ttyUSB[0-9]*" GROUP="dialout" MODE="0666"
```

Users need to be in group dialout to use Bus Pirate
```bash
sudo /sbin/usermod -a -G dialout <user>
```

Connect to bus pirate in terminal mode
```bash
screen /dev/ttyUSB0 115200
```


#### Bus Pirate Firmware Update
We are using community firmware 7.0 compiled on 18.09.2016 for Bus Pirate v3.
See [Bus Pirate community repository](https://github.com/mikebdp2/Bus_Pirate/tree/master/package_latest/BPv3) for a precompiled package.

To check which version the board is running connect to it through UART and type "i" in the command prompt
```
$ screen 115200 /dev/ttyUSB0

HiZ>i
Bus Pirate v3.5
Community Firmware v7.0 - goo.gl/gCzQnW [HiZ UART I2C SPI] Bootloader v4.4
DEVID:0x0447 REVID:0x3046 (24FJ64GA002 B8)
http://dangerousprototypes.com
```

Set Bus Pirate to bootloader mode in terminal application (command: $), or short PGC and PGD pins while you connect the board, keeping them shorted, then

```
./pirate-loader --dev=/dev/ttyUSB0 --hex=bpv3_fw7.0_optS_18092016.hex

Writing page 41 row 335, a780...OK
Erasing page 42, a800...ERROR [50]

Error updating firmware :(
```

The error is related to a bug in the BP bootloader, but the firmware should be updated successfully.
If you don't get the error, then the board has an updated bootloader.


#### Useful Documentation
[Bitbang mode](http://dangerousprototypes.com/docs/Bitbang)<br/>
[I2C binary mode](http://dangerousprototypes.com/docs/I2C_%28binary%29)
