#include "sampa.hpp"

const uint8_t Sampa::REG_HWADD    = 0x00;
const uint8_t Sampa::REG_VACFG    = 0x0d;
const uint8_t Sampa::REG_CMD      = 0x0e;
const uint8_t Sampa::REG_ADCDEL   = 0x10;
const uint8_t Sampa::REG_ADCTRIM  = 0x11;
const uint8_t Sampa::REG_SOCFG    = 0x12;
const uint8_t Sampa::REG_SODRVST  = 0x13;
const uint8_t Sampa::REG_BYPASS   = 0x1f;
const uint8_t Sampa::REG_RINGCNT  = 0x21;
const uint8_t Sampa::REG_CLKCONF  = 0x22;
const uint8_t Sampa::REG_BOUNDARY = 0x23;
const uint8_t Sampa::REG_CHEN0    = 0x24;
const uint8_t Sampa::REG_CHEN1    = 0x25;
const uint8_t Sampa::REG_CHEN2    = 0x26;
const uint8_t Sampa::REG_CHEN3    = 0x27;

const std::map<uint8_t, Sampa::reg_def_t>
Sampa::registers_ = {
  {REG_HWADD,    std::make_tuple("HWADD"   )},
  {REG_VACFG,    std::make_tuple("VACFG"   )},
  {REG_CMD,      std::make_tuple("CMD"     )},
  {REG_ADCDEL,   std::make_tuple("ADCDEL"  )},
  {REG_ADCTRIM,  std::make_tuple("ADCTRIM" )},
  {REG_SOCFG,    std::make_tuple("SOCFG"   )},
  {REG_SODRVST,  std::make_tuple("SODRVST" )},
  {REG_BYPASS,   std::make_tuple("BYPASS"  )},
  {REG_RINGCNT,  std::make_tuple("RINGCNT" )},
  {REG_CLKCONF,  std::make_tuple("CLKCONF" )},
  {REG_BOUNDARY, std::make_tuple("BOUNDARY")},
  {REG_CHEN0,    std::make_tuple("CHEN0"   )},
  {REG_CHEN1,    std::make_tuple("CHEN1"   )},
  {REG_CHEN2,    std::make_tuple("CHEN2"   )},
  {REG_CHEN3,    std::make_tuple("CHEN3"   )}
};


void
Sampa::dumpGlobalRegisters(std::ostream& os)
{
  os << std::setfill('0') << std::hex;
  for (auto& reg : registers_)
    os << "  [0x" << std::setw(2) << static_cast<uint32_t>(reg.first) << "]"
       << "  " << std::left << std::setw(10) << std::setfill('.') << std::get<0>(reg.second)
       << std::right << std::setfill('0')
       << " : 0x" << std::setw(2) << static_cast<uint32_t>(i2c_.readByte(reg.first))
       << "\n";
  os << std::setfill(' ') << std::dec;
}
