#pragma once

#include <map>
#include "gbt_sca_i2c.hpp"

class Sampa
{
  private:
    gbt::ScaI2c& i2c_;

    using reg_def_t = std::tuple<const std::string>;

  public:
    Sampa(gbt::ScaI2c& i2c) : i2c_(i2c) {}

    /** Map defining the SAMPA registers
     */
    static const std::map<uint8_t, reg_def_t> registers_;

    /** Sampa register definitions & constants
     */
    static const uint8_t REG_HWADD;
    static const uint8_t REG_VACFG;
    static const uint8_t REG_CMD;
    static const uint8_t REG_ADCDEL;
    static const uint8_t REG_ADCTRIM;
    static const uint8_t REG_SOCFG;
    static const uint8_t REG_SODRVST;
    static const uint8_t REG_BYPASS;
    static const uint8_t REG_RINGCNT;
    static const uint8_t REG_CLKCONF;
    static const uint8_t REG_BOUNDARY;
    static const uint8_t REG_CHEN0;
    static const uint8_t REG_CHEN1;
    static const uint8_t REG_CHEN2;
    static const uint8_t REG_CHEN3;

    /** Dump SAMPA global registers
     *  @param os Output stream to dump to
     */
    void dumpGlobalRegisters(std::ostream& os = std::cout);

    /** Read register via I2C
     *  @param addr I2C register address
     *  @return Read value
     */
    inline uint8_t readRegister(uint32_t addr) const { return i2c_.readByte(addr); }

    /** Write register via I2C
     *  @param addr I2C register address
     *  @param value Write value
     *  @return Read-back value
     */
    inline uint8_t writeRegister(uint32_t addr, uint8_t value) const { return i2c_.writeByte(addr, value); }

    /** Get SAMPA clock config
     */
    inline uint8_t getClkConfig() const { return readRegister(REG_CLKCONF); }
};
