-- ------------------------------------------------------------------------------
-- Record types for HDLC core
--
-- kirsch@fias.uni-frankfurt.de
-- ------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

package gbt_hdlc_light_types is

  --
  -- Records grouped by IN/OUT
  type t_gbt_hdlc_ctrl is record
    tx_payload : std_logic_vector(63 downto 0);
    tx_start   : std_logic;                      -- Start Tx (implicitly enables reception of one frame)
    rx_start   : std_logic;                      -- Start Rx (explicity enable reception of one frame)
    tx_mode    : std_logic_vector(1 downto 0);   -- Regular packet (00), Svl CONNECT (01), Svl RESET (10), Svl TEST (11)
    tx_addr    : std_logic_vector(7 downto 0);   -- Only for regular packets
  end record;

  type t_gbt_hdlc_stat is record
    tx_rdy     : std_logic;
    tx_seq_nr  : std_logic_vector(7 downto 0);
    tx_fsm     : std_logic_vector(3 downto 0);
    rx_payload : std_logic_vector(63 downto 0);  -- Rx HDLC payload
    rx_addr    : std_logic_vector(7 downto 0);   -- Rx HDLC address field
    rx_ctrl    : std_logic_vector(7 downto 0);   -- Rx HDLC control field
    rx_rdy     : std_logic;                      -- Frame received
    rx_svl_cmd : std_logic;                      -- Last frame was supervisory level command
    rx_crc     : std_logic_vector(31 downto 0);
    rx_seq_nr  : std_logic_vector(7 downto 0);
    rx_fsm     : std_logic_vector(3 downto 0);
  end record;


  --
  -- Records grouped by IN/OUT and clock domains
  type t_gbt_hdlc_ctrl_tx is record
    tx_payload : std_logic_vector(63 downto 0);
    tx_start   : std_logic;                      -- Start Tx (implicitly enables reception of one frame)
    tx_mode    : std_logic_vector(1 downto 0);   -- Regular packet (00), Svl CONNECT (01), Svl RESET (10), Svl TEST (11)
    tx_addr    : std_logic_vector(7 downto 0);   -- Only for regular packets
  end record;

  type t_gbt_hdlc_ctrl_rx is record
    rx_start   : std_logic;                      -- Start Rx (explicity enable reception of one frame)
  end record;

  type t_gbt_hdlc_stat_tx is record
    tx_rdy     : std_logic;
    tx_seq_nr  : std_logic_vector(7 downto 0);
    tx_fsm     : std_logic_vector(3 downto 0);
  end record;

  type t_gbt_hdlc_stat_rx is record
    rx_payload : std_logic_vector(63 downto 0);  -- Rx HDLC payload
    rx_addr    : std_logic_vector(7 downto 0);   -- Rx HDLC address field
    rx_ctrl    : std_logic_vector(7 downto 0);   -- Rx HDLC control field
    rx_rdy     : std_logic;                      -- Frame received
    rx_svl_cmd : std_logic;                      -- Last frame was supervisory level command
    rx_crc     : std_logic_vector(31 downto 0);
    rx_seq_nr  : std_logic_vector(7 downto 0);
    rx_fsm     : std_logic_vector(3 downto 0);
  end record;

end gbt_hdlc_light_types;
