-- ------------------------------------------------------------------------------
-- Simple HDLC light core wrapper
--   with record interface
--
-- kirsch@fias.uni-frankfurt.de
-- ------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

library work;
use work.gbt_hdlc_light_types.all;

entity gbt_hdlc_light_wrap is
  port (
    tx_clk    : in std_logic;
    tx_rst    : in std_logic;
    rx_clk    : in std_logic;
    rx_rst    : in std_logic;

    hdlc_ctrl_tx : in  t_gbt_hdlc_ctrl_tx;
    hdlc_ctrl_rx : in  t_gbt_hdlc_ctrl_rx;
    hdlc_stat_tx : out t_gbt_hdlc_stat_tx;
    hdlc_stat_rx : out t_gbt_hdlc_stat_rx;

    rx_2b     : in  std_logic_vector(1 downto 0);
    tx_2b     : out std_logic_vector(1 downto 0)
  );
end gbt_hdlc_light_wrap;

architecture arc of gbt_hdlc_light_wrap is

begin

  inst_0 : entity work.gbt_hdlc_light
    port map (
      -- clk             : in std_logic;    -- FIXME: (temporary) port to two clock domains for
      -- rst             : in std_logic;    --          use in current G-RORC
      tx_clk => tx_clk,
      tx_rst => tx_rst,
      rx_clk => rx_clk,
      rx_rst => rx_rst,

      hdlc_tx_payload => hdlc_ctrl_tx.tx_payload,
      hdlc_tx_start   => hdlc_ctrl_tx.tx_start,
      hdlc_rx_start   => hdlc_ctrl_rx.rx_start,
      hdlc_tx_mode    => hdlc_ctrl_tx.tx_mode,
      hdlc_tx_addr    => hdlc_ctrl_tx.tx_addr,
      hdlc_tx_rdy     => hdlc_stat_tx.tx_rdy,
      hdlc_tx_seq_nr  => hdlc_stat_tx.tx_seq_nr,
      hdlc_tx_fsm     => hdlc_stat_tx.tx_fsm,
      hdlc_rx_payload => hdlc_stat_rx.rx_payload,
      hdlc_rx_addr    => hdlc_stat_rx.rx_addr,
      hdlc_rx_ctrl    => hdlc_stat_rx.rx_ctrl,
      hdlc_rx_rdy     => hdlc_stat_rx.rx_rdy,
      hdlc_rx_svl_cmd => hdlc_stat_rx.rx_svl_cmd,
      hdlc_rx_crc     => hdlc_stat_rx.rx_crc,
      hdlc_rx_seq_nr  => hdlc_stat_rx.rx_seq_nr,
      hdlc_rx_fsm     => hdlc_stat_rx.rx_fsm,
      rx_2b           => rx_2b,
      tx_2b           => tx_2b
    );

end arc;
