-- ------------------------------------------------------------------------------
-- Simple HDLC sw core wrapper
--   with record interface
--
-- kirsch@fias.uni-frankfurt.de
-- ------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

library work;
use work.gbt_hdlc_sw_types.all;

entity gbt_hdlc_sw_wrap is
  port (
    tx_clk    : in std_logic;
    rx_clk    : in std_logic;
    tx_rst    : in std_logic;
    rx_rst    : in std_logic;

    hdlc_ctrl_tx : in  t_gbt_hdlc_sw_ctrl_tx;
    hdlc_ctrl_rx : in  t_gbt_hdlc_sw_ctrl_rx;
    hdlc_stat_tx : out t_gbt_hdlc_sw_stat_tx;
    hdlc_stat_rx : out t_gbt_hdlc_sw_stat_rx;

    rx_2b     : in  std_logic_vector(1 downto 0);
    tx_2b     : out std_logic_vector(1 downto 0)
  );
end gbt_hdlc_sw_wrap;

architecture arc of gbt_hdlc_sw_wrap is

begin

  inst_0 : entity work.gbt_hdlc_sw
    generic map (
      SHREG_WIDTH => HDLC_SW_SHREG_WIDTH
    )
    port map (
      tx_clk => tx_clk,
      rx_clk => rx_clk,
      tx_rst => tx_rst,
      rx_rst => rx_rst,
      tx_usr_rst => hdlc_ctrl_tx.tx_usr_rst,
      rx_usr_rst => hdlc_ctrl_rx.rx_usr_rst,

      tx_data     => hdlc_ctrl_tx.tx_frame,
      rx_data     => hdlc_stat_rx.rx_frame,
      --
      tx_start       => hdlc_ctrl_tx.tx_start,
      tx_done        => hdlc_stat_tx.tx_done,
      tx_clr         => hdlc_ctrl_tx.tx_clr,
      tx_bitswap_sel => hdlc_ctrl_tx.tx_bitswap_sel,
      tx_sw_seq_inc  => hdlc_ctrl_tx.tx_sw_seq_inc,
      rx_ack         => hdlc_ctrl_rx.rx_ack,
      rx_clr         => hdlc_ctrl_rx.rx_clr,
      rx_done        => hdlc_stat_rx.rx_done,
      rx_bitswap_sel => hdlc_ctrl_rx.rx_bitswap_sel,
      rx_sw_seq_inc  => hdlc_ctrl_rx.rx_sw_seq_inc,
      --
      tx_seq_cnt     => hdlc_stat_tx.tx_seq_cnt,
      rx_seq_cnt     => hdlc_stat_rx.rx_seq_cnt,
      tx_sw_seq_cnt  => hdlc_stat_tx.tx_sw_seq_cnt,
      rx_sw_seq_cnt  => hdlc_stat_rx.rx_sw_seq_cnt,
      rx_sof_cnt     => hdlc_stat_rx.rx_sof_cnt,
      --
      rx_2b     => rx_2b,
      tx_2b     => tx_2b
    );

end arc;
