# HDLC Core Simulation

Simulation for the cores
- `gbt_hdlc_light`
- `gbt_hdlc_sw`

The core `gbt_sca_verilog`, provided by the CERN microelectronics group, serves as
reference and data sink/source.

## HDLC Light

### Testbench
The HDLC light core is tested by sending HDLC frames to each other. To test compatiblity with the
reference core, and to obtain a received data set serving as reference, each core type sends/receives
HDLC frames from the its own type and to/from the other type.

The testbench therefore features:
- two instances of the `gbt_sca_verilog` reference implementation with wrappers, DUT0 and DUT1, and
- two instances of the `hdlc_light` core, DUT2 and DUT3.

They are wired in a way that 
- DUT0 (`gbt_sca_verilog`) Tx is connected to Rx of DUT1 (`gbt_sca_verilog`) and DUT2 (`hdlc_light`), and
- DUT2 (`hdlc_light`) Tx is connected to Rx of DUT0 (`gbt_sca_verilog`) and DUT3 (`hdlc_light`).

Identical stimuli are applied to DUT0 and DUT2 scripts. 

#### Running the Simulation
The script `sim_hdlc_light.sh` is used to run the simulation, see `-h | --help` for options.
It allows to run ModelSim in GUI or command-line mode, and triggers comparison
of the result files in the latter.

#### Simulation Output
The simulation produces a number of output files:
- `LOG_sca_requests.txt`<br/>
   Chronological list of the SCA request stimuli. 
- `DUT*_frames_serial.txt`<br/>
   HDLC frame data in serial form as transmitted via the GBT link, including bitstuffing.
- `DUT*_rx.txt`<br/>
   Received SCA frame data, which forms the basis for a comparison of the operation of the cores.
   The script `compare_results.sh` can be used to quickly diff these result files.
- `DBG*_*x.txt`, if `-d | --debug`<br/>
   Detailed simulation output containing the relevant shift registers, control flags and FSM states.


## HDLC Sw

### Testbench
The HDLC Sw core is sending to, and receiving from, the `gbt_sca_verilog` core.

### Running the Simulation
The scripts `sim_hdlc_sw.sh` is used to run the simulation, see `-h | --help` for details.

### Simulation output
- `SIMSW_DUT0_rx_frame.txt`<br/>
   Raw content of the `rx_data` register when frame reception is signalled with `rx_done`.
- `SIMSW_REF0_rx_data.txt`<br/>
   Content of the `DATARX[63..0]` core register when frame reception is signalled with `irq`.
