#!/bin/bash

#
# Testbench
#
#    Stimulus 
#        |
#        |--->  DUT0 (sca_verilog) -----tx-------> DUT1 (sca_verilog)
#        |                                   |
#        |                                   |---> DUT2 (hdlc_light)
#        |        
#        |--->  DUT2 (hdlc_light) ------tx-------> DUT0 (sca_verilog)
#                                            |
#                                            |---> DUT3 (hdlc_light)
#
#  DUT0,1 : gbt_sca_verilog instances (with wrappers)
#  DUT2,3 : hdlc_light instances
#
#  Identical stimuli to DUT0 and DUT2
#     (DUT0 will transmit identical frame twice for some reason)
#


# Input files
F0="DUT0_sca_verilog_rx.txt"
F1="DUT1_sca_verilog_rx.txt"
F2="DUT2_hdlc_light_rx.txt"
F3="DUT3_hdlc_light_rx.txt"


# Command line parsing
for key in "$@"
do
  case $key in
    --keep)
      KEEP_TMP_FILES=1
      ;;
    -h | --help)
      echo "Options:"
      echo "  -h | --help   Show this help"
      echo "  --keep        Skip cleanup and keep temporary files"
      exit 0
      ;;
  esac
  shift
done


# Temporary files
F0_tmp=$(mktemp "/tmp/_`basename $F0 .txt`.XXXXXX")
F1_tmp=$(mktemp "/tmp/_`basename $F1 .txt`.XXXXXX")
F2_tmp=$(mktemp "/tmp/_`basename $F2 .txt`.XXXXXX")
F3_tmp=$(mktemp "/tmp/_`basename $F3 .txt`.XXXXXX")


# DUT0: Select data field only
echo "Reformatting $F0 -> $F0_tmp"
cat $F0 | cut -d ' ' -f 1 > $F0_tmp

# DUT1: Drop double frames received from DUT0, select data field only
#sed -e '1~2d' $F1 | cut -d ' ' -f 1 > $F1_tmp
#cat $F1 | awk 'NR % 2 == 0' | cut -d ' ' -f 1 > $F1_tmp
#
# Edit: Double frames are gone with revised interfacing of
#   Pippo's wrappers and gbt_sca_verilog core
echo "Reformatting $F1 -> $F1_tmp"
cat $F1 | cut -d ' ' -f 1 > $F1_tmp

# DUT2: Kick out lines containing the supervisory level commands
echo "Reformatting $F2 -> $F2_tmp"
sed -e '/^SVL/ d' $F2 > $F2_tmp

# DUT3: Kick out lines containing the supervisory level commands
echo "Reformatting $F3 -> $F3_tmp"
sed -e '/^SVL/ d' $F3 > $F3_tmp



echo ""
echo "------------------------------------------------------------------"
echo "- COMPARING"
echo "-   hdlc_light  -> sca_verilog* ($F0) vs."
echo "-   sca_verilog -> sca_verilog* ($F1)"
echo "------------------------------------------------------------------"
diff -qs $F0_tmp $F1_tmp
echo ""
diff -N -U0 $F0_tmp $F1_tmp


echo ""
echo "------------------------------------------------------------------"
echo "- COMPARING"
echo "-  hdlc_light  -> sca_verilog* ($F0) vs."
echo "-  sca_verilog -> hdlc_light*  ($F2)"
echo "------------------------------------------------------------------"
diff -qs $F0_tmp $F2_tmp
echo ""
diff -N -U0 $F0_tmp $F2_tmp


echo ""
echo "------------------------------------------------------------------"
echo "- COMPARING"
echo "-  sca_verilog -> hdlc_light*  ($F2)"
echo "-  hdlc_light  -> hdlc_light*  ($F3)"
echo "------------------------------------------------------------------"
diff -qs $F2 $F3
echo ""
diff -N -U0 $F2 $F3


echo ""
echo "------------------------------------------------------------------"
echo "- COMPARING"
echo "-  sca_verilog -> sca_verilog*  ($F1)"
echo "-  hdlc_light  -> hdlc_light*  ($F3)"
echo "------------------------------------------------------------------"
diff -qs $F1_tmp $F3_tmp
echo ""
diff -N -U0 $F1_tmp $F3_tmp


if [ -z $KEEP_TMP_FILES ]; then
  echo ""
  echo "------------------------------------------------------------------"
  echo "- Cleaning up"
  echo "------------------------------------------------------------------"
  rm -v $F0_tmp
  rm -v $F1_tmp
  rm -v $F2_tmp
  rm -v $F3_tmp
fi
