-- --------------------------------------------------------------------------------
--   HDLC light core testbench
-- --------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.bus_pkg.all;
use work.gbt_hdlc_sw_types.all;

entity tb_sw is
end tb_sw;

architecture behavioral of tb_sw is
  -- CONSTANTs
  constant c_CLK_PERIOD_40  : time      := 25 ns;  -- 40 MHz

  -- TB SIGNAL
  signal clk_40          : std_logic := '1';
  signal rst_40          : std_logic;

  signal dut0_sw_ctrl_tx : t_gbt_hdlc_sw_ctrl_tx := (
       tx_start => '0', tx_clr => '0', tx_bitswap_sel => '1',
       tx_sw_seq_inc => '0', tx_usr_rst => '0',
       tx_frame => (others => '0'), others => '0' );

  signal dut0_sw_ctrl_rx : t_gbt_hdlc_sw_ctrl_rx := (
       rx_ack => '0', rx_clr => '0', rx_bitswap_sel => '1',
       rx_sw_seq_inc => '0', rx_usr_rst => '0');

  signal dut0_sw_stat_tx : t_gbt_hdlc_sw_stat_tx;
  signal dut0_sw_stat_rx : t_gbt_hdlc_sw_stat_rx;
  signal dut0_sw_rx_2b   : std_logic_vector(1 downto 0);
  signal dut0_sw_tx_2b   : std_logic_vector(1 downto 0);

  signal ref0_in       : sca_in_rec;
  signal ref0_out      : sca_out_rec;
  signal ref0_rx_infst : gbt_sc_rx_rec;
  signal ref0_tx_infst : gbt_sc_tx_rec;

  signal s0_sca_datai  : std_logic_vector(31 downto 0) := (others => '0');
  signal s0_sca_addr   : std_logic_vector(2 downto 0) := (others => '0');
  signal s0_sca_wr     : std_logic := '0';
  signal s0_sca_datao  : std_logic_vector(31 downto 0);
  signal s0_sca_irq    : std_logic;
  signal s0_sca_irq_r  : std_logic;

begin

  --
  --
  REF0 : entity work.gbt_sca_w
    port map (
      gbt_sca_rx_i => ref0_rx_infst,
      gbt_sca_tx_i => ref0_tx_infst,

      sca_i        => ref0_in,
      sca_o        => ref0_out,

      ecrx_i       => dut0_sw_tx_2b,
      ectx_o       => dut0_sw_rx_2b
    );

  ref0_rx_infst.clk   <= clk_40;
  ref0_tx_infst.clk   <= clk_40;
  ref0_rx_infst.reset <= rst_40;
  ref0_tx_infst.reset <= rst_40;

  -- Keep naming so we can reuse sim functions :/
  ref0_in.data <= s0_sca_datai;
  ref0_in.add  <= s0_sca_addr;
  ref0_in.wr_p <= s0_sca_wr;
  s0_sca_datao <= ref0_out.data;
  s0_sca_irq   <= ref0_out.irq;

  process(clk_40)
  begin
    if rising_edge(clk_40) then
      s0_sca_irq_r <= s0_sca_irq;
    end if;
  end process;

  --
  --
  DUT0 : entity work.gbt_hdlc_sw_wrap
    port map (
      tx_clk => clk_40,
      rx_clk => clk_40,
      tx_rst => rst_40,
      rx_rst => rst_40, 

      hdlc_ctrl_tx => dut0_sw_ctrl_tx,
      hdlc_ctrl_rx => dut0_sw_ctrl_rx,
      hdlc_stat_tx => dut0_sw_stat_tx,
      hdlc_stat_rx => dut0_sw_stat_rx,

      rx_2b  => dut0_sw_rx_2b,
      tx_2b  => dut0_sw_tx_2b
    );

  -- Auto-acknowledge rx_frames for DUT0
  process(clk_40)
  begin 
    if rising_edge(clk_40) then
      if dut0_sw_stat_rx.rx_done = '1' then
        dut0_sw_ctrl_rx.rx_ack <= '1';
      else
        dut0_sw_ctrl_rx.rx_ack <= '0';
      end if;
    end if;
  end process;


  --
  -- Clock & reset
  clk_40 <= not clk_40 after (c_CLK_PERIOD_40 / 2.0);

  process
  begin
    -- Assert Reset
    rst_40 <= '1';
    wait for (c_CLK_PERIOD_40 * 100);
    rst_40 <= '0';
    wait;
  end process;

end behavioral;
