#!/bin/bash

# Command line parsing
VSIM="vsim -c"
VSIM_GUI=0
DEBUG=0
LONG=0
SEED=0
COMPARE=1
TEST=0
CREATE_ONLY=0

for key in "$@"
do
  case $key in
    -g | --gui)
      VSIM_GUI=1
      VSIM=vsim
      ;;
    -l | --long)
      LONG=1
      ;;
      -d | --debug)
      DEBUG=1
      ;;
    -s | --seed)
      SEED=$2
      shift
      ;;
    --no-compare)
      COMPARE=0
      ;;
    --test)
      TEST=1
      ;;
    --clean)
      if [ -e "transcript" ]; then rm -vf transcript; fi 
      if [ -e "hdlc_light.cr.mti" ]; then rm -vf hdlc_light.cr.mti; fi 
      if [ -e "vsim.wlf" ]; then rm -vf vsim.wlf; fi 
      if [ -e "work" ]; then rm -vrf work; fi
      if [ -e "hdlc_light.mpf" ]; then rm -vf hdlc_light.mpf; fi
      exit 0;
      ;;
    --create)
      CREATE_ONLY=1
      ;;
    -h | --help)
echo "\
Run HDLC light core simulation

Options
  -g | --gui    Run ModelSim with GUI
  -l | --long   Run test with large number of sequences (takes forever)
  -d | --debug  Enable extended debugging
  -s | --seed   Give random number generator seed
  --no-compare  Don't run diff script
  --clean       Clean previous sim from directory (and exit)
  --create      Run only until after project creation step (and exit)
  -h | --help   Show this help
"
exit 0
      ;;
  esac
  shift
done

CMD_CREATE_PROJECT="VSIM_GUI=0 $VSIM -do prj_hdlc_light.tcl"
CMD="DEBUG=$DEBUG VSIM_GUI=$VSIM_GUI LONG=$LONG SEED=$SEED $VSIM -do run_hdlc_light.tcl"

hash vsim 2>/dev/null || {
  echo >&2 "Unable to find ModelSim binary (vsim). Aborting."; exit 1;
}


# Create project if not there
if [ ! -e "hdlc_light.mpf" ]; then
  echo "Creating project \"$CMD_CREATE_PROJECT\""
  eval $CMD_CREATE_PROJECT
fi

if [ x"$CREATE_ONLY" == x"1" ]; then
  exit 0
fi


# Run simulation
echo "\
Running simulation
  $CMD
"
if [ x"$TEST" == x"0" ]; then
  eval $CMD
  echo "End of simulation"
fi


# Check for sim errors and compare results
if [[ `tail -n 1 transcript` == "# Errors: 0,"* ]]; then
  if [ x"$COMPARE" == x"1" ]; then
    echo ""
    ./compare_result.sh
  fi
else
  echo ""
  echo "Simulation error. Aborting."
  exit 2
fi

