#!/bin/bash

# Command line parsing
VSIM="vsim -c"
VSIM_GUI=0
DEBUG=0
CREATE_ONLY=0
TEST=0

PRJ=hdlc_sw

for key in "$@"
do
  case $key in
    -g | --gui)
      VSIM_GUI=1
      VSIM=vsim
      ;;
    --clean)
      if [ -e "transcript" ]; then rm -vf transcript; fi 
      if [ -e "${PRJ}cr.mti" ]; then rm -vf ${PRJ}.cr.mti; fi 
      if [ -e "vsim.wlf" ]; then rm -vf vsim.wlf; fi 
      if [ -e "work" ]; then rm -vrf work; fi
      if [ -e "${PRJ}.mpf" ]; then rm -vf ${PRJ}.mpf; fi
      exit 0;
      ;;
    --create)
      CREATE_ONLY=1
      ;;
    -h | --help)
echo "\
Run ${PRJ} core simulation

Options
  -g | --gui    Run ModelSim with GUI
  --clean       Clean previous sim from directory (and exit)
  --create      Run only until after project creation step (and exit)
  -h | --help   Show this help
"
exit 0
      ;;
  esac
  shift
done

CMD_CREATE_PROJECT="VSIM_GUI=0 $VSIM -do prj_${PRJ}.tcl"
CMD="DEBUG=$DEBUG VSIM_GUI=$VSIM_GUI $VSIM -do run_${PRJ}.tcl"

hash vsim 2>/dev/null || {
  echo >&2 "Unable to find ModelSim binary (vsim). Aborting."; exit 1;
}


# Create project if not there
if [ ! -e "${PRJ}.mpf" ]; then
  echo "Creating project \"$CMD_CREATE_PROJECT\""
  eval $CMD_CREATE_PROJECT
fi

if [ x"$CREATE_ONLY" == x"1" ]; then
  exit 0
fi


# Run simulation
echo "\
Running simulation
  $CMD
"
if [ x"$TEST" == x"0" ]; then
  eval $CMD
  echo "End of simulation"
fi
