set DEBUG_HDLC_LIGHT_RX 0
set DEBUG_HDLC_LIGHT_TX 0

catch {if {$env(DEBUG) == 1} {
  set DEBUG_HDLC_LIGHT_RX 1
  set DEBUG_HDLC_LIGHT_TX 1
} }

set ITERATIONS_RND 5
set ITERATIONS_BITSTUFF 3

catch {if {$env(LONG) == 1} {
  set ITERATIONS_RND 2000
  set ITERATIONS_BITSTUFF -1
} }

set RND_SEED 0

catch {if {$env(SEED) != 0} {
  set RND_SEED $env(SEED)
} }

#-------------------------------------------------------------------------------
onbreak {resume}
catch {nowhen *}

set NumericStdNoWarnings 1
when {$now = @51ns } {set NumericStdNoWarnings 0}

do func.tcl
do func_test.tcl

quietly set testlog [open "LOG_sca_requests.txt" "w"]
quietly set dut0_fp [open "DUT0_sca_verilog_rx.txt" "w"]
quietly set dut1_fp [open "DUT1_sca_verilog_rx.txt" "w"]
quietly set dut2_fp [open "DUT2_hdlc_light_rx.txt" "w"]
quietly set dut3_fp [open "DUT3_hdlc_light_rx.txt" "w"]

when -label packet_received_DUT0 {\
  (sim:/tb/s0_sca_irq == '1') && (sim:/tb/s0_sca_irq_r == '0') && \
  sim:/tb/s_clk_40'event && (sim:/tb/s_clk_40 == '1')} {
  global dut0_fp
  dump_sca_verilog_rx_data $dut0_fp "sim:/tb/DUT0"
}

when -label packet_received_DUT1 {\
  (sim:/tb/s1_sca_irq == '1') && (sim:/tb/s1_sca_irq_r == '0') && \
  sim:/tb/s_clk_40'event && (sim:/tb/s_clk_40 == '1')} {
  global dut1_fp
  dump_sca_verilog_rx_data $dut1_fp "sim:/tb/DUT1"
}

# Trigger on rising edge of hdlc_rx_rdy
when -label packet_received_HDLC0 {\
  (sim:/tb/DUT2/hdlc_rx_rdy == '1') && (sim:/tb/gsl_hdlc_rcv_rdy_0_r == '0') &&
  sim:/tb/s_clk_40'event && (sim:/tb/s_clk_40 == '1')} {
  global dut2_fp
  dump_hdlc_light_rx_data $dut2_fp "sim:/tb/DUT2"
}

# Trigger on rising edge of hdlc_rx_rdy
when -label packet_received_HDLC1 {\
  (sim:/tb/DUT3/hdlc_rx_rdy == '1') && (sim:/tb/gsl_hdlc_rcv_rdy_1_r == '0') &&
  sim:/tb/s_clk_40'event && (sim:/tb/s_clk_40 == '1')} {
  global dut3_fp
  dump_hdlc_light_rx_data $dut3_fp "sim:/tb/DUT3"
}


if {$DEBUG_HDLC_LIGHT_RX != 0} {
  quietly set dbg2_fp [open "DBG_DUT2_rx.txt" "w"]
  dump_dbg_hdlc_rx_header $dbg2_fp
  when -label debug_rx_DUT2 {\
    sim:/tb/DUT2/rx_clk'event && (sim:/tb/DUT2/rx_clk == '1') && \
    ((sim:/tb/DUT2/cstate_rx /= SR_IDLE) || \
    ((sim:/tb/DUT2/nstate_rx == SR_WAIT_BEGIN) && (sim:/tb/DUT2/cstate_rx == SR_IDLE))) \
    } {
    global dbg2_fp
    dump_dbg_hdlc_rx $dbg2_fp "sim:/tb/DUT2"
  }

  quietly set dbg3_fp [open "DBG_DUT3_rx.txt" "w"]
  dump_dbg_hdlc_rx_header $dbg3_fp
  when -label debug_rx_DUT3 {\
    sim:/tb/DUT3/rx_clk'event && (sim:/tb/DUT3/rx_clk == '1') && \
    ((sim:/tb/DUT3/cstate_rx /= SR_IDLE) || \
    ((sim:/tb/DUT3/nstate_rx == SR_WAIT_BEGIN) && (sim:/tb/DUT3/cstate_rx == SR_IDLE))) \
    } {
    global dbg3_fp
    dump_dbg_hdlc_rx $dbg3_fp "sim:/tb/DUT3"
  }
}


if {$DEBUG_HDLC_LIGHT_TX != 0} {
  quietly set dbg2tx_fp [open "DBG_DUT2_tx.txt" "w"]
  #dump_dbg_hdlc_tx_header $dbg2tx_fp
  when -label debug_tx_DUT2 {\
    sim:/tb/DUT2/tx_clk'event && (sim:/tb/DUT2/tx_clk == '1') && \
    ((sim:/tb/DUT2/cstate_tx /= SS_IDLE) && (sim:/tb/DUT2/cstate_tx /= SS_RESET)) \
    } {
    global dbg2tx_fp
    dump_dbg_hdlc_tx $dbg2tx_fp "sim:/tb/DUT2"
  }
}

run 4 us
test_rnd $testlog $ITERATIONS_RND $RND_SEED
test_svl_cmds $testlog
test_bitstuff $testlog $ITERATIONS_BITSTUFF
test_rnd $testlog $ITERATIONS_RND
test_bitstuff $testlog $ITERATIONS_BITSTUFF
run 10 us

close $testlog
close $dut0_fp
close $dut1_fp
close $dut2_fp
close $dut3_fp

catch {close $dbg2_fp}
catch {close $dbg2tx_fp}
catch {close $dbg3_fp}
