#-------------------------------------------------------------------------------
onbreak {resume}
catch {nowhen *}

set NumericStdNoWarnings 1
when {$now = @51ns } {set NumericStdNoWarnings 0}

do func.tcl
set tb_path "sim:/tb_sw"

proc dump_sw_rx_frame {fp {core_path "sim:/tb/DUT0"}} {
  # Dump the frame data received by the gbt_sca_verilog core to a text file
  set d [string tolower [examine -hex $core_path/hdlc_stat_rx.rx_frame]]
  puts $fp "DATA=0x$d"
}


#-------------------------------------------------------------------------------
quietly set dut0_fp_rx [open "SIMSW_DUT0_rx_frame.txt" "w"]
quietly set ref0_fp_rx [open "SIMSW_REF0_rx_data.txt" "w"]

# Trigger on rising edge of hdlc_rx_rdy
when -label rx_frame_DUT0 {\
  sim:/tb_sw/clk_40'event && (sim:/tb_sw/clk_40 == '1') && 
  (sim:/tb_sw/DUT0/hdlc_stat_rx.rx_done == '1') } {
  global dut0_fp_rx
  global tb_path
  dump_sw_rx_frame $dut0_fp_rx "$tb_path/DUT0"
}

when -label packet_received_REF0 {\
  (sim:/tb_sw/s0_sca_irq == '1') && (sim:/tb_sw/s0_sca_irq_r == '0') && \
  sim:/tb_sw/clk_40'event && (sim:/tb_sw/clk_40 == '1')} {
  global ref0_fp_rx
  global tp_path
  dump_sca_verilog_rx_data $ref0_fp_rx "$tb_path/REF0"
}


#-------------------------------------------------------------------------------
run 4 us

set cycles 400
set sca_length 0x0
set sca_trid 0xab
set sca_command 0x21
set sca_channel 0x2
set sca_data 0xdeadabab


# Frame data (include bitstuffing & CRC) for hdlc_sw was extracted from TX of gbt_sca_verilog
#   in sim and matches the given stimulus for gbt_sca_verilog
hdlc_sw_send_frame 0x7f7f7f7f7f7f7f7f7f7f7f7f7f7f7eecb200000000addeabab210002ab00007e
sca_send_request 0 $sca_trid $sca_channel $sca_length $sca_command $sca_data
run [expr ($cycles+1)*$clk_period_ns] ns


set sca_length 0x1

hdlc_sw_send_frame 0x7f7f7f7f7f7f7f7f7f7f7f7f7f7f7ecbcf00000000addeabab210102ab02007e
sca_send_request 0 $sca_trid $sca_channel $sca_length $sca_command $sca_data
run [expr ($cycles+2)*$clk_period_ns] ns


set sca_length 0x2

hdlc_sw_send_frame 0x7f7f7f7f7f7f7f7f7f7f7f7f7f7f7ea24800000000addeabab210202ab04007e
sca_send_request 0 $sca_trid $sca_channel $sca_length $sca_command $sca_data
run [expr ($cycles+3)*$clk_period_ns] ns


set sca_length 0x3

hdlc_sw_send_frame 0x7f7f7f7f7f7f7f7f7f7f7f7f7f7f7e853500000000addeabab210302ab06007e
sca_send_request 0 $sca_trid $sca_channel $sca_length $sca_command $sca_data
run [expr ($cycles+4)*$clk_period_ns] ns


set sca_length 0x4

hdlc_sw_send_frame 0x7f7f7f7f7f7f7f7f7f7f7f7f7e70b8000000000000addeabab210402ab08007e
sca_send_request 0 $sca_trid $sca_channel $sca_length $sca_command $sca_data
run [expr ($cycles+5)*$clk_period_ns] ns


#-------------------------------------------------------------------------------
close $dut0_fp_rx
close $ref0_fp_rx
