## Software
`hdlc.hpp` implements a number of common routines to support to the `gbt_hdlc_sw` core. These include,
amongst others, (de-)bitstuffing of the buffer content, (de-)composition of HDLC frames and filling
of unused buffer space with an appropriate inter-frame fill pattern. It uses `boost::multiprecision`
to perform operations on the underlying buffer, which is transmitted/received by `gbt_hdlc_sw` using
a shift register.

See, e.g., the [T-RORC utils repository](https://gitlab.cern.ch/hengel/alice-tpc-trorc-utils) for an
application example.


### Notes
- *Exception thrown when debitstuffing frame received from GBT-SCA, complaining about abort marker*

    The following pattern is found in the buffer when the exceptions is thrown:<br/>
  `01111110_11111` with `11111` being the five MSBs of the FCS field and `01111110` being the frame end flag.<br/>

    The HDLC standard (ISO/IEC 13239:2002(E)) says the following:<br/>
  > 4.3.1 Synchronous transmission
  > The transmitter shall examine the frame content between the two flag sequences including the address,
  > control and FCS fields, as well as the frame format and HCS fields, when present, and shall insert a
  > "0" bit after all sequences of 5 contiguous "1" bits (including the last 5 bits of the FCS) to ensure
  > that a flag sequence is not simulated. The receiver shall examine the frame content and shall discard
  > any "0" bit which directly follows 5 contiguous "1" bits.
  
    The observed behavior points to an issue with the GBT-SCA, as one would expect an additional bitstuffed '0'
    between FCS and end flag in the case above and similar cases.<br/>
    Currently, applications using this software work around the issue by implementing a frame retransmit on
    the software level.
